#!/bin/bash

# Default architecture
DEFAULT_ARCH="linux/amd64"

# Supported architectures (adjust as needed)
ARCHITECTURES=("linux/amd64" "linux/arm64" "linux/arm/v7" "darwin/amd64" "darwin/arm64")

# Ensure all necessary directories exist and go modules are ready
prepare_build() {
    # Create necessary directories if they don't exist
    mkdir -p dist
    mkdir -p build_logs
    mkdir -p test_logs  # Directory for test logs

    # Initialize go modules if go.mod does not exist
    if [ ! -f go.mod ]; then
        echo "Initializing Go modules"
        go mod init ssh-timeout  # Replace 'yourmodule' with your actual module name or path
    fi

    # Fetch and ensure all dependencies are up to date
    echo "Checking dependencies..."
    go mod tidy
}

# Build function
build_binary() {
    os=$1
    arch=$2
    output_name="ssh-timeout"

    if [[ "$os/$arch" != "$DEFAULT_ARCH" ]]; then
        output_name="${output_name}_${os}_${arch}"
    fi

    output_name="dist/${output_name}"

    echo "Building for ${os}/${arch} -> ${output_name}"
    GOOS=${os} GOARCH=${arch} go build -o ${output_name} main.go 2>build_logs/${os}_${arch}_build.log
    if [ $? -eq 0 ]; then
        echo "Successfully built ${output_name}"
    else
        echo "Failed to build ${output_name}. Check build_logs/${os}_${arch}_build.log for errors."
    fi
}

# Test function
run_tests() {
    if ls *.go | grep '_test.go$' >/dev/null; then
        echo "Running tests..."
        go test ./... > test_logs/test_output.log 2>&1
        if [ $? -eq 0 ]; then
            echo "Tests completed successfully."
        else
            echo "Tests failed. Check test_logs/test_output.log for details."
        fi
    else
        echo "No test files found, skipping tests."
    fi
}

# Main Build Process
prepare_build
run_tests  # Run tests optionally before the build
for arch in "${ARCHITECTURES[@]}"; do
    IFS='/' read -r -a parts <<< "$arch"  # Split architecture string
    os=${parts[0]}
    arch=${parts[1]}
    build_binary $os $arch
done

echo "Build process completed."

